@@ -9,7 +9,8 @@ import click
 from tqdm import tqdm
 
 from splitgraph.__version__ import __version__
-from splitgraph.config import CONFIG, SG_CMD_ASCII, get_singleton
+from splitgraph.config import CONFIG, SG_CMD_ASCII
+from splitgraph.config.config import get_singleton
 from splitgraph.config.management import patch_and_save_config
 from splitgraph.exceptions import DockerUnavailableError, EngineSetupError
 from splitgraph.utils.docker import copy_to_container, get_docker_client
@@ -455,7 +456,7 @@ def configure_engine_c(name):
 @click.argument("name", default=DEFAULT_ENGINE)
 def version_engine_c(name):
     """Get version of Splitgraph engine."""
-    from splitgraph.engine import get_engine
+    from splitgraph.engine.config import get_engine
 
     if name == DEFAULT_ENGINE:
         engine = get_engine()
@@ -485,7 +486,7 @@ def upgrade_engine_c(ctx, image, no_pull, name):
     metadata volumes intact), creating a container based on a newer
     image and finally reinitializing the engine to perform needed migrations.
     """
-    from splitgraph.engine import get_engine
+    from splitgraph.engine.config import get_engine
 
     # Get reference to engine to extract its connection params
     if name == DEFAULT_ENGINE:
