@@ -29,6 +29,7 @@ from splitgraph.config import (
 from splitgraph.core.indexing.bloom import filter_bloom_index, generate_bloom_index
 from splitgraph.core.indexing.range import filter_range_index, generate_range_index
 from splitgraph.core.metadata_manager import MetadataManager, Object
+from splitgraph.core.overlay import SG_ROW_SEQ, WRITE_UPPER_PREFIX
 from splitgraph.core.types import Changeset, TableSchema
 from splitgraph.engine import ResultShape
 from splitgraph.engine.postgres.engine import (
@@ -204,6 +205,11 @@ class Digest:
             tuple((left - right) & 0xFFFF for left, right in zip(self.shorts, other.shorts))
         )
 
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, Digest):
+            return NotImplemented
+        return self.hex() == other.hex()
+
     def __neg__(self) -> "Digest":
         return Digest(tuple(-v & 0xFFFF for v in self.shorts))
 
@@ -429,7 +435,7 @@ class FragmentManager(MetadataManager):
             # Store the fragment in a temporary location and then find out its hash and rename to the actual target.
             # Optimisation: in the future, we can hash the upserted rows that we need preemptively and possibly
             # avoid storing the object altogether if it's a duplicate.
-            tmp_object_id = self._store_changeset(
+            tmp_table_id = self._store_changeset(
                 sub_changeset, table_name, schema, table.table_schema
             )
 
@@ -439,42 +445,22 @@ class FragmentManager(MetadataManager):
                 object_id,
                 rows_inserted,
                 rows_deleted,
-            ) = self._get_patch_fragment_hashes_stats(sub_changeset, table, tmp_object_id)
+            ) = self._get_patch_fragment_hashes_stats(sub_changeset, table, tmp_table_id)
 
             object_ids.append(object_id)
 
-            # Wrap this rename in a SAVEPOINT so that if the table already exists,
-            # the error doesn't roll back the whole transaction (us creating and registering all other objects).
-            with self.object_engine.savepoint("object_rename"):
-                source_query = SQL("SELECT * FROM {}.{}").format(
-                    Identifier("pg_temp"), Identifier(tmp_object_id)
-                )
-
-                if in_fragment_order:
-                    source_query += SQL(" ") + self._get_order_by_clause(
-                        in_fragment_order, table.table_schema
-                    )
-
-                try:
-                    self.object_engine.store_object(
-                        object_id=object_id,
-                        source_query=source_query,
-                        schema_spec=add_ud_flag_column(table.table_schema),
-                        overwrite=overwrite,
-                    )
-                except UniqueViolation:
-                    # Someone registered this object (perhaps a concurrent pull) already.
-                    logging.info(
-                        "Object %s for table %s/%s already exists, continuing...",
-                        object_id,
-                        table.repository,
-                        table.table_name,
-                    )
-                self.object_engine.delete_table("pg_temp", tmp_object_id)
-                # There are some cases where an object can already exist in the object engine (in the cache)
-                # but has been deleted from the metadata engine, so when it's recreated, we'll skip
-                # actually registering it. Hence, we still want to proceed trying to register
-                # it no matter what.
+            self._store_object_from_temp_table(
+                object_id,
+                tmp_table_id,
+                table,
+                in_fragment_order=in_fragment_order,
+                overwrite=overwrite,
+            )
+            self.object_engine.delete_table("pg_temp", tmp_table_id)
+            # There are some cases where an object can already exist in the object engine (in the cache)
+            # but has been deleted from the metadata engine, so when it's recreated, we'll skip
+            # actually registering it. Hence, we still want to proceed trying to register
+            # it no matter what.
 
             # Same here: if we are being called as part of a commit and an object
             # already exists, we'll roll back everything that the caller has done
@@ -517,6 +503,52 @@ class FragmentManager(MetadataManager):
         object_id = "o" + sha256((content_hash + schema_hash).encode("ascii")).hexdigest()[:-2]
         return deletion_hash, insertion_hash, object_id, rows_inserted, rows_deleted
 
+    def _store_object_from_temp_table(
+        self,
+        object_id: str,
+        tmp_table_id: str,
+        table: "Table",
+        in_fragment_order: Optional[List[str]] = None,
+        overwrite: bool = False,
+    ) -> None:
+        """
+        Persist an object by reading from a temporary source table.
+
+        :param object_id: The id of the object to be stored.
+        :param tmp_table_id: Temporary source table from which to load data.
+        :param table: Table object pointing to the current HEAD table.
+        :param in_fragment_order: Key to sort data inside each chunk by.
+        :param overwrite: Overwrite physical objects that already exist.
+        :return:
+        """
+        # Wrap this rename in a SAVEPOINT so that if the table already exists,
+        # the error doesn't roll back the whole transaction (us creating and registering all other objects).
+        with self.object_engine.savepoint("object_rename"):
+            source_query = SQL("SELECT * FROM {}.{}").format(
+                Identifier("pg_temp"), Identifier(tmp_table_id)
+            )
+
+            if in_fragment_order:
+                source_query += SQL(" ") + self._get_order_by_clause(
+                    in_fragment_order, table.table_schema
+                )
+
+            try:
+                self.object_engine.store_object(
+                    object_id=object_id,
+                    source_query=source_query,
+                    schema_spec=add_ud_flag_column(table.table_schema),
+                    overwrite=overwrite,
+                )
+            except UniqueViolation:
+                # Someone registered this object (perhaps a concurrent pull) already.
+                logging.info(
+                    "Object %s for table %s/%s already exists, continuing...",
+                    object_id,
+                    table.repository,
+                    table.table_name,
+                )
+
     def _store_changeset(
         self, sub_changeset: Any, table: str, schema: str, table_schema: TableSchema
     ) -> str:
@@ -582,6 +614,8 @@ class FragmentManager(MetadataManager):
         :param new_schema_spec: New schema of the table (use the old table's schema by default).
         :param split_changeset: See `Repository.commit` for reference
         :param extra_indexes: Dictionary of {index_type: column: index_specific_kwargs}.
+        :param in_fragment_order: Key to sort data inside each chunk by.
+        :param overwrite: Overwrite physical objects that already exist.
         """
 
         # TODO does the reasoning in the docstring actually make sense? If the point is to, say, for a query
@@ -641,6 +675,165 @@ class FragmentManager(MetadataManager):
                 [(image_hash, old_table.table_name, new_schema_spec, old_table.objects)],
             )
 
+    def record_overlay_table_as_patch(
+        self,
+        old_table: "Table",
+        schema: str,
+        image_hash: str,
+        new_schema_spec: TableSchema = None,
+        extra_indexes: Optional[ExtraIndexInfo] = None,
+        in_fragment_order: Optional[List[str]] = None,
+        overwrite: bool = False,
+    ) -> None:
+        """
+        Flushes the pending changes from the staging (aka upper) table for a given overlay table (i.e. view) and records
+        them, registering the new objects.
+
+        :param old_table: Table object pointing to the current HEAD table (actually a view)
+        :param schema: Schema the table is checked out into.
+        :param image_hash: Image hash to store the table under
+        :param new_schema_spec: New schema of the table (use the old table's schema by default).
+        :param extra_indexes: Dictionary of {index_type: column: index_specific_kwargs}.
+        :param in_fragment_order: Key to sort data inside each chunk by.
+        :param overwrite: Overwrite physical objects that already exist.
+        """
+        upper_table = WRITE_UPPER_PREFIX + old_table.table_name
+        new_schema_spec = new_schema_spec or old_table.table_schema
+
+        pk_cols, non_pk_cols = self.object_engine.schema_spec_to_cols(new_schema_spec)
+        pk_cols_s = SQL(",").join(map(Identifier, pk_cols))
+
+        if len(non_pk_cols) > 0:
+            # If there are PKs defined on the table we want to obfuscate the non-PK fields of deleted records by setting
+            # them to NULLs.
+            target_cols = []
+            for c in new_schema_spec:
+                if c.is_pk:
+                    target_cols.append(Identifier(c.name))
+                else:
+                    target_cols.append(
+                        SQL("CASE WHEN {0} IS TRUE THEN {1} ELSE NULL END").format(
+                            Identifier(SG_UD_FLAG), Identifier(c.name)
+                        )
+                    )
+            target_cols = SQL(",").join(target_cols)
+        else:
+            target_cols = SQL(",").join(Identifier(c.name) for c in new_schema_spec)
+
+        # Remove redundant rows in the object
+        object_select = (
+            # Group by the primary key and order by the row version
+            # for conflicting rows
+            SQL("WITH _sg_flattened AS (SELECT *, ROW_NUMBER() OVER (PARTITION BY ")
+            + pk_cols_s
+            + SQL(" ORDER BY {0} DESC) AS _sg_row_order FROM {1}.{2}) SELECT ").format(
+                Identifier(SG_ROW_SEQ), Identifier(schema), Identifier(upper_table)
+            )
+            # Select the latest versions of every PK.
+            + target_cols
+            + SQL(", ")
+            + Identifier(SG_UD_FLAG)
+            + SQL("FROM _sg_flattened WHERE _sg_row_order = 1 ORDER BY {0} ASC").format(
+                Identifier(SG_ROW_SEQ)
+            )
+        )
+
+        if in_fragment_order:
+            object_select += SQL(" ") + self._get_order_by_clause(
+                in_fragment_order, new_schema_spec
+            )
+
+        tmp_object_id = get_temporary_table_id()
+        self.object_engine.store_object(
+            object_id=tmp_object_id,
+            source_query=object_select,
+            schema_spec=add_ud_flag_column(new_schema_spec),
+            overwrite=overwrite,
+        )
+
+        # Calculate the object ID, continue if there are deleted or inserted rows
+        schema_hash = self._calculate_schema_hash(new_schema_spec)
+
+        deletion_hash, rows_deleted = self.calculate_content_hash(
+            SPLITGRAPH_META_SCHEMA,
+            tmp_object_id,
+            new_schema_spec,
+            chunk_condition_sql=SQL("WHERE {} IS FALSE").format(Identifier(SG_UD_FLAG)),
+        )
+        insertion_hash, rows_inserted = self.calculate_content_hash(
+            SPLITGRAPH_META_SCHEMA,
+            tmp_object_id,
+            new_schema_spec,
+            chunk_condition_sql=SQL("WHERE {} IS TRUE").format(Identifier(SG_UD_FLAG)),
+        )
+
+        if rows_inserted != 0 or rows_deleted != 0:
+            logging.debug(
+                "Persisting %s/%s inserted/deleted rows with hash %s/%s for overlay table %s.",
+                rows_inserted,
+                rows_deleted,
+                insertion_hash.hex(),
+                deletion_hash.hex(),
+                old_table.table_name,
+            )
+            content_hash = (insertion_hash - deletion_hash).hex()
+
+            object_id = "o" + sha256((content_hash + schema_hash).encode("ascii")).hexdigest()[:-2]
+
+            # Rename the object to its actual ID
+            try:
+                self.object_engine.rename_object(
+                    old_object_id=tmp_object_id,
+                    new_object_id=object_id,
+                )
+            finally:
+                self.object_engine.delete_objects([tmp_object_id])
+
+            with self.metadata_engine.savepoint("object_register"):
+                try:
+                    # Register the object: this also indexes it and (in the case of our custom object
+                    # manager) marks it as external/uploads it to S3.
+                    self._register_object(
+                        object_id,
+                        namespace=old_table.repository.namespace,
+                        insertion_hash=insertion_hash.hex(),
+                        deletion_hash=deletion_hash.hex(),
+                        table_schema=new_schema_spec,
+                        extra_indexes=extra_indexes,
+                        rows_inserted=rows_inserted,
+                        rows_deleted=rows_deleted,
+                    )
+                except UniqueViolation:
+                    logging.info(
+                        "Object %s already exists, continuing...",
+                        object_id,
+                    )
+
+            self.register_tables(
+                old_table.repository,
+                [
+                    (
+                        image_hash,
+                        old_table.table_name,
+                        new_schema_spec,
+                        old_table.objects + [object_id],
+                    )
+                ],
+            )
+
+        # Truncate the upper table to prepare for the next write
+        self.object_engine.run_sql(
+            SQL("TRUNCATE TABLE {}.{}").format(Identifier(schema), Identifier(upper_table))
+        )
+
+        # Finally, point the LQ FDW to the latest image to pick up the new changes after truncating the upper table
+        self.object_engine.run_sql(
+            SQL("ALTER SERVER {} OPTIONS (SET image_hash %s)").format(
+                Identifier(old_table.repository.lq_server_name(schema)),
+            ),
+            (image_hash,),
+        )
+
     def split_changeset_boundaries(
         self, changeset: Changeset, change_key: List[Tuple[str, str]], objects: List[str]
     ) -> List[Changeset]:
@@ -723,7 +916,7 @@ class FragmentManager(MetadataManager):
         table_schema: Optional[TableSchema] = None,
         chunk_condition_sql: Optional[Composable] = None,
         chunk_condition_args: Optional[List[Any]] = None,
-    ) -> Tuple[str, int]:
+    ) -> Tuple[Digest, int]:
         """
         Calculates the homomorphic hash of table contents.
 
@@ -753,7 +946,7 @@ class FragmentManager(MetadataManager):
         )
 
         return (
-            reduce(operator.add, (Digest.from_memoryview(r) for r in row_digests)).hex(),
+            reduce(operator.add, (Digest.from_memoryview(r) for r in row_digests), Digest.empty()),
             len(row_digests),
         )
 
@@ -811,7 +1004,9 @@ class FragmentManager(MetadataManager):
             tmp_object_id,
             table_schema,
         )
-        object_id = "o" + sha256((content_hash + schema_hash).encode("ascii")).hexdigest()[:-2]
+        object_id = (
+            "o" + sha256((content_hash.hex() + schema_hash).encode("ascii")).hexdigest()[:-2]
+        )
         try:
             # Check if the object metadata and the object files exist. Sometimes we might have
             # one but not the other, in which case we want to get out of this invalid state.
@@ -841,7 +1036,7 @@ class FragmentManager(MetadataManager):
             self._register_object(
                 object_id,
                 namespace=namespace,
-                insertion_hash=content_hash,
+                insertion_hash=content_hash.hex(),
                 deletion_hash="0" * 64,
                 table_schema=table_schema,
                 extra_indexes=extra_indexes,
@@ -962,7 +1157,7 @@ class FragmentManager(MetadataManager):
         # We need to do multiple things here in a specific way to not tank the performance:
         #  * Chunk the table up ordering by PK (or potentially another chunk key in the future)
         #  * Run LTHash on added rows in every chunk
-        #  * Copy each chunk into a CStore table (adding ranges/bloom filter intex to it
+        #  * Copy each chunk into a CStore table (adding ranges/bloom filter index to it
         #    in the metadata).
         #
         # Chunking is very slow to do with
